#-----------------------------------------------------------------------------
# Version: 1.0.0
# Compatible: SketchUp 7 (PC)
#             (other versions untested)
#-----------------------------------------------------------------------------
#
# Thomas Thomassen
# thomas[at]thomthom[dot]net
#
#-----------------------------------------------------------------------------

require 'sketchup.rb'

#-----------------------------------------------------------------------------


module TT_Comp_Dropper
	
	unless file_loaded?('tt_component_dropper.rb')
		plugins_menu = UI.menu('Tools')
		plugins_menu.add_item('Component Dropper')	{ self.activate_tool }
	end
	
	def self.activate_tool
		my_tool = CompDropper.new
		Sketchup.active_model.tools.push_tool(my_tool)
	end

	class CompDropper
	
		def initialize
			@picked = nil
			
			path = 'Plugins/TT_CompDropper'
			c_dropper		= Sketchup.find_support_file('dropper.png', path)
			c_dropper_err	= Sketchup.find_support_file('dropper_invalid.png', path)
			@c_dropper		= (c_dropper) ? UI.create_cursor(c_dropper, 2, 29) : 0
			@c_dropper_err	= (c_dropper_err) ? UI.create_cursor(c_dropper_err, 2, 29) : 0
		end
		
		def activate
			@picked = nil
			@pos = [0,0,0]
			@drawn = false
			Sketchup::set_status_text 'Pick component to replace the selected.'
		end
		
		def deactivate(view)
			view.invalidate if @drawn
		end
		
		def onMouseMove(flags, x, y, view)
			ph = view.pick_helper
			ph.do_pick(x, y)
			
			if ph.best_picked != @picked
				@pos = [x,y,0]
				@picked = ph.best_picked
				view.invalidate
			end
		end
		
		def onLButtonUp(flags, x, y, view)
			if is_instance(@picked)
				model = Sketchup.active_model
				definition = get_definition(@picked)
				
				TT_Comp_Dropper.start_operation('Component Dropper')
				
				new_sel = []
				model.selection.each { |e|
					next unless is_instance(e)
					e_def = get_definition(e)
					next if e_def == definition
					
					# Get transformation without the scaling
					sx = e_def.bounds.width	/ e.bounds.width
					sy = e_def.bounds.height	/ e.bounds.height
					sz = e_def.bounds.depth	/ e.bounds.depth
					ts = Geom::Transformation.scaling(sx, sy, sz)
					t = e.transformation * ts
					
					# Create new scaling
					sx = e.bounds.width  / definition.bounds.width
					sy = e.bounds.height / definition.bounds.height
					sz = e.bounds.depth  / definition.bounds.depth
					ts = Geom::Transformation.scaling(sx, sy, sz)
					
					# Replace instance
					if e.is_a?(Sketchup::ComponentInstance)
						e.definition = definition
						e.transformation = t * ts
					else
						ents = e.parent.entities
						new_sel << ents.add_instance(definition, t * ts)
						e.erase!
					end
				}
				model.selection.add(new_sel)
				
				model.commit_operation
				#model.tools.pop_tool
			end
		end
		
		def draw(view)
			if is_instance(@picked)
				definition = get_definition(@picked)
				view.draw_text(@pos, definition.name)
				@drawn = true
			end
		end
		
		# Draw the geometry
		def draw_geometry(pt1, pt2, view)
			view.draw_line(pt1, pt2)
		end
		
		def onSetCursor
			if is_instance(@picked)
				UI.set_cursor(@c_dropper)
			else
				UI.set_cursor(@c_dropper_err)
			end
		end
		
		def is_instance(entity)
			return entity.is_a?(Sketchup::ComponentInstance) || entity.is_a?(Sketchup::Group)
		end
		
		def get_definition(instance)
			# ComponentInstance
			return instance.definition if instance.is_a?(Sketchup::ComponentInstance)
			# Group
			if instance.entities.parent.instances.include?(instance)
				return instance.entities.parent
			else
				Sketchup.active_model.definitions.each { |definition|
					return definition if definition.instances.include?(instance)
				}
			end
			return nil # Error. We should never exit here.
		end

	end # class
	
	### HELPER METHODS ### ---------------------------------------------------
	
	def self.start_operation(name)
		model = Sketchup.active_model
		# Make use of the SU7 speed boost with start_operation while
		# making sure it works in SU6.
		if Sketchup.version.split('.')[0].to_i >= 7
			model.start_operation(name, true)
		else
			model.start_operation(name)
		end
	end
	
end # module

file_loaded('tt_component_dropper.rb')